@@ -474,8 +474,8 @@ class WirelessAdapter(object):
         if_up_cmd = "{} {}".format(IF_UP_CMD, self.wlan_if_name)
 
         try:
-            self.file_logger.error("Taking interface down...")
-            if_bounce = subprocess.check_output(if_down_cmd, stderr=subprocess.STDOUT, shell=True).decode()
+            self.file_logger.warning("Taking interface down...")
+            subprocess.check_output(if_down_cmd, stderr=subprocess.STDOUT, shell=True).decode()
         except subprocess.CalledProcessError as exc:
             output = exc.output.decode()
             error_descr = "i/f down command appears to have failed. Error: {} (signalling error)".format(str(output))
@@ -486,8 +486,8 @@ class WirelessAdapter(object):
         time.sleep(10)
 
         try:
-            self.file_logger.error("Bringing interface up...")
-            if_bounce = subprocess.check_output(if_up_cmd, stderr=subprocess.STDOUT, shell=True).decode()
+            self.file_logger.warning("Bringing interface up...")
+            subprocess.check_output(if_up_cmd, stderr=subprocess.STDOUT, shell=True).decode()
         except subprocess.CalledProcessError as exc:
             output = exc.output.decode()
             error_descr = "i/f up command appears to have failed. Error: {} (signalling error)".format(str(output))
